@@ -884,7 +884,7 @@ def test_expiry():
     assert detail["time"] - start_time > 0.09
 
 
-def test_equality():
+def test_equality(tmpdir):
     """Test sane behaviour for equality and hashing.
 
     Make sure that different CachingFileSystem only test equal to each other
@@ -897,9 +897,11 @@ def test_equality():
     from fsspec.implementations.local import LocalFileSystem
 
     lfs = LocalFileSystem()
-    cfs1 = CachingFileSystem(fs=lfs, cache_storage="raspberry")
-    cfs2 = CachingFileSystem(fs=lfs, cache_storage="banana")
-    cfs3 = CachingFileSystem(fs=lfs, cache_storage="banana")
+    dir1 = f"{tmpdir}/raspberry"
+    dir2 = f"{tmpdir}/banana"
+    cfs1 = CachingFileSystem(fs=lfs, cache_storage=dir1)
+    cfs2 = CachingFileSystem(fs=lfs, cache_storage=dir2)
+    cfs3 = CachingFileSystem(fs=lfs, cache_storage=dir2)
     assert cfs1 == cfs1
     assert cfs1 != cfs2
     assert cfs1 != cfs3
@@ -922,3 +924,28 @@ def test_str():
     lfs = LocalFileSystem()
     cfs = CachingFileSystem(fs=lfs)
     assert "CachingFileSystem" in str(cfs)
+
+
+def test_getitems_errors(tmpdir):
+    tmpdir = str(tmpdir)
+    os.makedirs(os.path.join(tmpdir, "afolder"))
+    open(os.path.join(tmpdir, "afile"), "w").write("test")
+    open(os.path.join(tmpdir, "afolder", "anotherfile"), "w").write("test2")
+    m = fsspec.get_mapper("file://" + tmpdir)
+    assert m.getitems(["afile", "bfile"], on_error="omit") == {"afile": b"test"}
+
+    # my code
+    m2 = fsspec.get_mapper("simplecache::file://" + tmpdir)
+    assert m2.getitems(["afile"], on_error="omit") == {"afile": b"test"}  # works
+    assert m2.getitems(["afile", "bfile"], on_error="omit") == {
+        "afile": b"test"
+    }  # throws KeyError
+
+    with pytest.raises(KeyError):
+        m.getitems(["afile", "bfile"])
+    out = m.getitems(["afile", "bfile"], on_error="return")
+    assert isinstance(out["bfile"], KeyError)
+    m = fsspec.get_mapper("file://" + tmpdir, missing_exceptions=())
+    assert m.getitems(["afile", "bfile"], on_error="omit") == {"afile": b"test"}
+    with pytest.raises(FileNotFoundError):
+        m.getitems(["afile", "bfile"])
