@@ -1,10 +1,11 @@
 import json
+import os
 
 import pytest
 
 import fsspec
 from fsspec.implementations.local import LocalFileSystem
-from fsspec.implementations.reference import _unmodel_hdf5
+from fsspec.implementations.reference import ReferenceFileSystem, ReferenceNotReachable
 from fsspec.tests.conftest import data, realfile, reset_files, server, win  # noqa: F401
 
 
@@ -23,6 +24,19 @@ def test_simple(server):  # noqa: F811
     assert fs.cat("b") == data[:5]
     assert fs.cat("c") == data[1 : 1 + 5]
     assert fs.cat("d") == b"hello"
+    with fs.open("d", "rt") as f:
+        assert f.read(2) == "he"
+
+
+def test_target_options(m):
+    m.pipe("data/0", b"hello")
+    refs = {"a": ["memory://data/0"]}
+    fn = "memory://refs.json.gz"
+    with fsspec.open(fn, "wt", compression="gzip") as f:
+        json.dump(refs, f)
+
+    fs = fsspec.filesystem("reference", fo=fn, target_options={"compression": "gzip"})
+    assert fs.cat("a") == b"hello"
 
 
 def test_ls(server):  # noqa: F811
@@ -34,6 +48,9 @@ def test_ls(server):  # noqa: F811
     assert {"name": "c", "type": "directory", "size": 0} in fs.ls("", detail=True)
     assert fs.find("") == ["a", "b", "c/d"]
     assert fs.find("", withdirs=True) == ["a", "b", "c", "c/d"]
+    assert fs.find("c", detail=True) == {
+        "c/d": {"name": "c/d", "size": 6, "type": "file"}
+    }
 
 
 def test_info(server):  # noqa: F811
@@ -75,6 +92,41 @@ def test_mutable(server, m):
     assert fs.cat("aa") == bin_data
 
 
+def test_put_get(tmpdir):
+    d1 = f"{tmpdir}/d1"
+    os.mkdir(d1)
+    with open(f"{d1}/a", "wb") as f:
+        f.write(b"1")
+    with open(f"{d1}/b", "wb") as f:
+        f.write(b"2")
+    d2 = f"{tmpdir}/d2"
+
+    fs = fsspec.filesystem("reference", fo={}, remote_protocol="file")
+    fs.put(d1, "out", recursive=True)
+
+    fs.get("out", d2, recursive=True)
+    assert open(f"{d2}/a", "rb").read() == b"1"
+    assert open(f"{d2}/b", "rb").read() == b"2"
+
+
+def test_put_get_single(tmpdir):
+    d1 = f"{tmpdir}/f1"
+    d2 = f"{tmpdir}/f2"
+    with open(d1, "wb") as f:
+        f.write(b"1")
+
+    # skip instance cache since this is the same kwargs as previous test
+    fs = fsspec.filesystem(
+        "reference", fo={}, remote_protocol="file", skip_instance_cache=True
+    )
+    fs.put_file(d1, "out")
+
+    fs.get_file("out", d2)
+    assert open(d2, "rb").read() == b"1"
+    fs.pipe({"hi": b"data"})
+    assert fs.cat("hi") == b"data"
+
+
 def test_defaults(server):  # noqa: F811
     refs = {"a": b"data", "b": (None, 0, 5)}
     fs = fsspec.filesystem(
@@ -89,22 +141,6 @@ def test_defaults(server):  # noqa: F811
     assert fs.cat("b") == data[:5]
 
 
-def test_inputs():  # noqa: F811
-    import io
-
-    refs = io.StringIO("""{"a": "data", "b": [null, 0, 5]}""")
-    fs = fsspec.filesystem(
-        "reference", fo=refs, target_protocol="http", target=realfile
-    )
-    assert fs.cat("a") == b"data"
-
-    refs = io.BytesIO(b"""{"a": "data", "b": [null, 0, 5]}""")
-    fs = fsspec.filesystem(
-        "reference", fo=refs, target_protocol="http", target=realfile
-    )
-    assert fs.cat("a") == b"data"
-
-
 jdata = """{
     "metadata": {
         ".zattrs": {
@@ -145,13 +181,6 @@ jdata = """{
 """
 
 
-def test_unmodel():
-    refs = _unmodel_hdf5(json.loads(jdata))
-    # apparently the output may or may not contain a space after ":"
-    assert b'"Conventions":"UGRID-0.9.0"' in refs[".zattrs"].replace(b" ", b"")
-    assert refs["adcirc_mesh/0"] == ("https://url", 8928, 8932)
-
-
 def test_spec1_expand():
     pytest.importorskip("jinja2")
     in_data = {
@@ -198,6 +227,29 @@ def test_spec1_expand():
     }
 
 
+def test_spec1_expand_simple():
+    pytest.importorskip("jinja2")
+    in_data = {
+        "version": 1,
+        "templates": {"u": "server.domain/path"},
+        "refs": {
+            "key0": "base64:ZGF0YQ==",
+            "key2": ["http://{{u}}", 10000, 100],
+            "key4": ["http://target_url"],
+        },
+    }
+    fs = fsspec.filesystem("reference", fo=in_data, target_protocol="http")
+    assert fs.references["key2"] == ["http://server.domain/path", 10000, 100]
+    fs = fsspec.filesystem(
+        "reference",
+        fo=in_data,
+        target_protocol="http",
+        template_overrides={"u": "not.org/p"},
+    )
+    assert fs.references["key2"] == ["http://not.org/p", 10000, 100]
+    assert fs.cat("key0") == b"data"
+
+
 def test_spec1_gen_variants():
     pytest.importorskip("jinja2")
     with pytest.raises(ValueError):
@@ -404,3 +456,168 @@ def test_cat_file_ranges(m):
     assert fs.cat_file("d", start=1) == other[4:10][1:]
     assert fs.cat_file("d", start=-5) == other[4:10][-5:]
     assert fs.cat_file("d", 1, -3) == other[4:10][1:-3]
+
+
+def test_cat_missing(m):
+    other = b"other test data"
+    m.pipe("/b", other)
+    fs = fsspec.filesystem(
+        "reference",
+        fo={
+            "c": ["memory://b"],
+            "d": ["memory://unknown", 4, 6],
+        },
+    )
+    with pytest.raises(FileNotFoundError):
+        fs.cat("notafile")
+
+    with pytest.raises(FileNotFoundError):
+        fs.cat(["notone", "nottwo"])
+
+    mapper = fs.get_mapper("")
+
+    with pytest.raises(KeyError):
+        mapper["notakey"]
+
+    with pytest.raises(KeyError):
+        mapper.getitems(["notone", "nottwo"])
+
+    with pytest.raises(ReferenceNotReachable) as ex:
+        fs.cat("d")
+    assert ex.value.__cause__
+    out = fs.cat("d", on_error="return")
+    assert isinstance(out, ReferenceNotReachable)
+
+    with pytest.raises(ReferenceNotReachable) as e:
+        mapper["d"]
+    assert '"d"' in str(e.value)
+    assert "//unknown" in str(e.value)
+
+    with pytest.raises(ReferenceNotReachable):
+        mapper.getitems(["c", "d"])
+
+    out = mapper.getitems(["c", "d"], on_error="return")
+    assert isinstance(out["d"], ReferenceNotReachable)
+
+    out = mapper.getitems(["c", "d"], on_error="omit")
+    assert list(out) == ["c"]
+
+
+def test_df_single(m):
+    pd = pytest.importorskip("pandas")
+    pytest.importorskip("pyarrow")
+    data = b"data0data1data2"
+    m.pipe({"data": data})
+    df = pd.DataFrame(
+        {
+            "path": [None, "memory://data", "memory://data"],
+            "offset": [0, 0, 4],
+            "size": [0, 0, 4],
+            "raw": [b"raw", None, None],
+        }
+    )
+    df.to_parquet("memory://stuff/refs.0.parq")
+    m.pipe(
+        ".zmetadata",
+        b"""{
+    "metadata": {
+        ".zgroup": {
+            "zarr_format": 2
+        },
+        "stuff/.zarray": {
+            "chunks": [1],
+            "compressor": null,
+            "dtype": "i8",
+            "filters": null,
+            "shape": [3],
+            "zarr_format": 2
+        }
+    },
+    "zarr_consolidated_format": 1,
+    "record_size": 10
+    }
+    """,
+    )
+    fs = ReferenceFileSystem(fo="memory:///", remote_protocol="memory")
+    allfiles = fs.find("")
+    assert ".zmetadata" in allfiles
+    assert ".zgroup" in allfiles
+    assert "stuff/2" in allfiles
+
+    assert fs.cat("stuff/0") == b"raw"
+    assert fs.cat("stuff/1") == data
+    assert fs.cat("stuff/2") == data[4:8]
+
+
+def test_df_multi(m):
+    pd = pytest.importorskip("pandas")
+    pytest.importorskip("pyarrow")
+    data = b"data0data1data2"
+    m.pipe({"data": data})
+    df0 = pd.DataFrame(
+        {
+            "path": [None, "memory://data", "memory://data"],
+            "offset": [0, 0, 4],
+            "size": [0, 0, 4],
+            "raw": [b"raw1", None, None],
+        }
+    )
+    df0.to_parquet("memory://stuff/refs.0.parq")
+    df1 = pd.DataFrame(
+        {
+            "path": [None, "memory://data", "memory://data"],
+            "offset": [0, 0, 2],
+            "size": [0, 0, 2],
+            "raw": [b"raw2", None, None],
+        }
+    )
+    df1.to_parquet("memory://stuff/refs.1.parq")
+    m.pipe(
+        ".zmetadata",
+        b"""{
+    "metadata": {
+        ".zgroup": {
+            "zarr_format": 2
+        },
+        "stuff/.zarray": {
+            "chunks": [1],
+            "compressor": null,
+            "dtype": "i8",
+            "filters": null,
+            "shape": [6],
+            "zarr_format": 2
+        }
+    },
+    "zarr_consolidated_format": 1,
+    "record_size": 3
+    }
+    """,
+    )
+    fs = ReferenceFileSystem(
+        fo="memory:///", remote_protocol="memory", skip_instance_cache=True
+    )
+    allfiles = fs.find("")
+    assert ".zmetadata" in allfiles
+    assert ".zgroup" in allfiles
+    assert "stuff/2" in allfiles
+    assert "stuff/4" in allfiles
+
+    assert fs.cat("stuff/0") == b"raw1"
+    assert fs.cat("stuff/1") == data
+    assert fs.cat("stuff/2") == data[4:8]
+    assert fs.cat("stuff/3") == b"raw2"
+    assert fs.cat("stuff/4") == data
+    assert fs.cat("stuff/5") == data[2:4]
+
+
+def test_mapping_getitems(m):
+    m.pipe({"a": b"A", "b": b"B"})
+
+    refs = {
+        "a": ["a"],
+        "b": ["b"],
+    }
+    h = fsspec.filesystem("memory")
+    fs = fsspec.filesystem("reference", fo=refs, fs=h)
+    mapping = fs.get_mapper("")
+    assert mapping.getitems(["b", "a"]) == {"a": b"A", "b": b"B"}
