@@ -29,7 +29,7 @@ def tempzip(data={}):
     finally:
         try:
             os.remove(f)
-        except (IOError, OSError):
+        except OSError:
             pass
 
 
@@ -194,7 +194,8 @@ def test_pickle_after_open_open():
 
 
 def test_mismatch():
-    with pytest.raises(ValueError, match="protocol"):
+    pytest.importorskip("s3fs")
+    with pytest.raises(ValueError):
         open_files(["s3://test/path.csv", "/other/path.csv"])
 
 
